/**
  * vue-formily-util v0.1.2
  *
  * @link https://github.com/vue-formily/util
  * @source https://github.com/vue-formily/util
  * (c) 2024 An Ha
  * @license MIT
  */
const _toString = Object.prototype.toString;
function isNullOrUndefined(value) {
    return value === null || value === undefined;
}
/**
 * Strict object type check. Only returns true
 * for plain JavaScript objects.
 */
function isPlainObject(value) {
    return _toString.call(value) === '[object Object]';
}
function isNumber(value) {
    return typeof value === 'number' && !isNaN(value);
}
function isString(value) {
    return typeof value === 'string';
}
function isUndefined(value) {
    return value === undefined;
}
function isPromise(promise) {
    return promise && typeof promise.then === 'function' && _toString.call(promise) === '[object Promise]';
}
/**
 * Check if the input string contains only numeric characters.
 */
function isNumeric(value) {
    const val = String(value);
    return val.length > 0 && !isNaN(+val);
}
function isFunction(fn) {
    return typeof fn === 'function';
}
/**
 * Returns true if the object is empty. Empty is defined as:
 * - null
 * - undefined
 * - a string with zero length
 * - an array with no elements
 * - a collection with no elements
 */
function isEmpty(value) {
    if (isNullOrUndefined(value)) {
        return true;
    }
    let length;
    if (Array.isArray(value) || isString(value)) {
        length = value.length;
    }
    if (isPlainObject(value)) {
        length = Object.keys(value).length;
    }
    return length === 0;
}
function isEqual(a, b) {
    if (a === b)
        return true;
    if (a instanceof Date && b instanceof Date && a.getTime() !== b.getTime()) {
        // If the values are Date, compare them as timestamps
        return false;
    }
    if (a !== Object(a) || b !== Object(b)) {
        // If the values aren't objects, they were already checked for equality
        return false;
    }
    const props = Object.keys(a);
    if (props.length !== Object.keys(b).length) {
        // Different number of props, don't bother to check
        return false;
    }
    return props.every(p => isEqual(a[p], b[p]));
}

/**
 * Merge objects or arrays
 */
function merge(target, ...sources) {
    return sources.reduce((acc, source) => {
        const keys = Object.keys(source || []);
        keys.forEach(key => {
            const item = source[key];
            if (isPlainObject(item)) {
                acc[key] = merge({}, acc[key], item);
            }
            else if (Array.isArray(item)) {
                acc[key] = merge([], acc[key], item);
            }
            else {
                const descriptor = Object.getOwnPropertyDescriptor(source, key);
                Object.defineProperty(acc, key, descriptor || item);
            }
        });
        return acc;
    }, target);
}
const rkey = /([^[\]]+)/g;
/**
 * Find the first found value in objects or arrays by its path
 */
function get(path, ...args) {
    const length1 = args.length;
    const paths = isString(path) ? path.split('.') : path;
    const length2 = paths.length;
    for (let i = 0; i < length1; i++) {
        let value = args[i];
        let j = -1;
        while (value && ++j < length2) {
            const fullName = paths[j];
            const m = fullName.match(rkey);
            if (m) {
                const length3 = m.length;
                for (let k = 0; k < length3; k++) {
                    const name = m[k];
                    if ((Array.isArray(value) || isPlainObject(value)) && name in value) {
                        value = value[name];
                        if (j === length2 - 1 && k === length3 - 1) {
                            return value;
                        }
                    }
                }
            }
        }
    }
    return undefined;
}
/**
 * Returns the index of the first element in the array that satisfies the provided testing function.
 */
function findIndex(arr = [], fn) {
    const length = arr.length;
    for (let i = 0; i < length; i++) {
        if (fn(arr[i], i, arr)) {
            return i;
        }
    }
    return -1;
}
/**
 * Creates a new array with all sub-array elements concatenated into it recursively up to the specified depth.
 */
function flatArray(arr, deep = Infinity) {
    return deep > 0 ? arr.reduce((acc, val) => acc.concat(Array.isArray(val) ? flatArray(val, deep - 1) : val), []) : arr;
}

const simpleInterpolator = /\{([^{}]+)\}/g;
function interpolate(format, data, interpolator = simpleInterpolator) {
    return format.replace(interpolator, (_, path) => {
        const value = get(path, ...flatArray([data]));
        return '' + value;
    });
}

export { findIndex, flatArray, get, interpolate, isEmpty, isEqual, isFunction, isNullOrUndefined, isNumber, isNumeric, isPlainObject, isPromise, isString, isUndefined, merge };
